################################################################################
# Danny Giusa Tyre Temperatures
# danny(dot)giusa(at)gmail(dot)com
# V1.3, 10.05.2014, AC Early Access 0.9.4
# 
# This little app provides informations about the state of your
# tyre like temperature, pressure and level of dirt
#
# Changelog:
#
# 1.0 
# - restructure of the techpreview app
# - added level of dirt with raising bar on tyrebar
#
# 1.1
# - added max temps and psi recording
# - rearrange dirtbars and value
#
# 1.2
# - added max values
# - fixed that under 25C drawn tyres disappear
# - using hue of HSV color wheel for temperature coloring (flitzi)
# - variables for optimal temperature and temperature range (flitzi)
# - added spinner for adjusting the optimal temperature while in the car (flitzi)
# - new alignment for the ui
#
# 1.3
# - added tyrewear (flitzi code)
# - added spinner for tyre compounds (Never_Eat_Yellow_Snow)
# - dirtlevel bar stil exist.
#
# 1.3.3 
# - removed app title
# - removed app logo
# - removed label compound
# - changed app size
# - changed spinner and tyre compound type position
#
################################################################################

import math
import ac
import acsys
import struct                          
import mmap                            

TYREINFO = 0
###appsize
x_app_size = 280
y_app_size = 160
###startingpoint of first label
x_start = 5
y_start = 10
###spaces from left to right label/tyre and upper and bottom
x_space = 215
y_space = 75
###spaces for drawn tyre 
x_tyre_position = x_start + 53 
y_tyre_position = y_start + 5
###spaces for drawn tyre from left to right
x_tyre_space = 80
x_tyredirt_space = 3
###settings for drawn tyre
w_tyre = 23
h_tyre = 50
###settings for offset of drawn tyredirt
w_tyredirt = w_tyre -6


################## CHANGE BACKROUND HERE ################## 
background = 0
################## CHANGE BACKROUND HERE ##################

tyres = [
    ("Street", 75, 85),
    ("Semislicks", 75, 100),
    ("GT2 Soft/S-Soft", 90, 105),
    ("GT2 Medium", 85, 105),
    ("GT2 Hard/S-Hard", 80, 100),
    ("GT3/Hypercars Softs", 80, 110),
    ("GT3/Hypercars Mediums", 75, 105),
    ("GT3/Hypercars Hard", 70, 100),
    ("Vintage F1 67", 50, 90),
]

class Tyre_Info:  

  def __init__(self, app, x, y):
    x_pressure = x
    x_dirt = x_pressure
    x_maxt = x + 80
    x_maxp = x_maxt
    
    y_pressure = y + 20
    y_dirt = y_pressure + 20
    y_maxt = y + 8
    y_maxp = y_maxt + 20
    
    self.optimal_temp = 100
    self.temp_range = 80
    
    self.maxtFL = 0
    self.maxtFR = 0
    self.maxtRL = 0
    self.maxtRR = 0
    
    self.maxpFL = 0
    self.maxpFR = 0
    self.maxpRL = 0
    self.maxpRR = 0
    
    self.tFLValue = ac.addLabel(app, "temp fl")
    ac.setPosition(self.tFLValue, x - 152, y)
    ac.setFontAlignment(self.tFLValue, "right")
    self.tFRValue = ac.addLabel(app, "temp fr")
    ac.setPosition(self.tFRValue, x + x_space, y)
    self.tRLValue = ac.addLabel(app, "temp rl")
    ac.setPosition(self.tRLValue, x - 152, y + y_space)
    ac.setFontAlignment(self.tRLValue, "right")
    self.tRRValue = ac.addLabel(app, "temp rr")
    ac.setPosition(self.tRRValue, x + x_space, y + y_space)
    
    self.pFLValue = ac.addLabel(app, "psi fl")
    ac.setPosition(self.pFLValue, x_pressure - 152, y_pressure)
    ac.setFontAlignment(self.pFLValue, "right")
    self.pFRValue = ac.addLabel(app, "psi fr")
    ac.setPosition(self.pFRValue, x_pressure + x_space, y_pressure)
    self.pRLValue = ac.addLabel(app, "psi rl")
    ac.setPosition(self.pRLValue, x_pressure - 152, y_pressure + y_space)
    ac.setFontAlignment(self.pRLValue, "right")
    self.pRRValue = ac.addLabel(app, "psi rr")
    ac.setPosition(self.pRRValue, x_pressure + x_space, y_pressure + y_space)
    
    self.wFLValue = ac.addLabel(app, "wear fl")
    ac.setPosition(self.wFLValue, x_dirt - 152, y_dirt)
    ac.setFontAlignment(self.wFLValue, "right")
    self.wFRValue = ac.addLabel(app, "wear fr")
    ac.setPosition(self.wFRValue, x_dirt + x_space, y_dirt)
    self.wRLValue = ac.addLabel(app, "wear rl")
    ac.setPosition(self.wRLValue, x_dirt - 152, y_dirt + y_space)
    ac.setFontAlignment(self.wRLValue, "right")
    self.wRRValue = ac.addLabel(app, "wear rr")
    ac.setPosition(self.wRRValue, x_dirt + x_space, y_dirt + y_space)
    
    #self.maxFont = ac.addLabel(app, "Max")
    #ac.setPosition(self.maxFont, 137, 28)
    #ac.setFontSize(self.maxFont, 12)
    #self.maxtFont = ac.addLabel(app, "C")
    #ac.setPosition(self.maxtFont, 145, 48)
    #ac.setFontSize(self.maxtFont, 12)
    #self.maxpFont = ac.addLabel(app, "psi")
    #ac.setPosition(self.maxpFont, 140, 68)
    #ac.setFontSize(self.maxpFont, 12)
    #self.maxtFontBottom = ac.addLabel(app, "C")
    #ac.setPosition(self.maxtFontBottom, 145, 123)
    #ac.setFontSize(self.maxtFontBottom, 12)
    #self.maxpFontBottom = ac.addLabel(app, "psi")
    #ac.setPosition(self.maxpFontBottom, 140, 143)
    #ac.setFontSize(self.maxpFontBottom, 12)
        
    self.maxtFLValue = ac.addLabel(app, "maxtemp fl")
    ac.setPosition(self.maxtFLValue, x_maxt, y_maxt)    
    ac.setFontSize(self.maxtFLValue, 12)
    ac.setFontAlignment(self.maxtFLValue, "left")    
    self.maxtFRValue = ac.addLabel(app, "maxtemp fr")
    ac.setPosition(self.maxtFRValue, x_maxt - 96, y_maxt)
    ac.setFontSize(self.maxtFRValue, 12)  
    ac.setFontAlignment(self.maxtFRValue, "right")    
    self.maxtRLValue = ac.addLabel(app, "maxtemp rl")
    ac.setPosition(self.maxtRLValue, x_maxt, y_maxt + y_space)
    ac.setFontSize(self.maxtRLValue, 12)
    ac.setFontAlignment(self.maxtRLValue, "left")    
    self.maxtRRValue = ac.addLabel(app, "maxtemp rr")
    ac.setPosition(self.maxtRRValue, x_maxt - 96, y_maxt + y_space)
    ac.setFontSize(self.maxtRRValue, 12)
    ac.setFontAlignment(self.maxtRRValue, "right")

    self.maxpFLValue = ac.addLabel(app, "maxpress fl")
    ac.setPosition(self.maxpFLValue, x_maxp, y_maxp)
    ac.setFontSize(self.maxpFLValue, 12)
    ac.setFontAlignment(self.maxpFLValue, "left")    
    self.maxpFRValue = ac.addLabel(app, "maxpress fr")
    ac.setPosition(self.maxpFRValue, x_maxp - 96, y_maxp)
    ac.setFontSize(self.maxpFRValue, 12)
    ac.setFontAlignment(self.maxpFRValue, "right")    
    self.maxpRLValue = ac.addLabel(app, "maxpress rl")
    ac.setPosition(self.maxpRLValue, x_maxp, y_maxp + y_space)
    ac.setFontSize(self.maxpRLValue, 12)
    ac.setFontAlignment(self.maxpRLValue, "left")    
    self.maxpRRValue = ac.addLabel(app, "maxpress rr")
    ac.setPosition(self.maxpRRValue, x_maxp - 96, y_maxp + y_space)
    ac.setFontSize(self.maxpRRValue, 12)
    ac.setFontAlignment(self.maxpRRValue, "right")
    
  def setTemp(self, tFL, tFR, tRL, tRR):
    ac.setText(self.tFLValue, "{0} C".format(round(tFL)))
    ac.setText(self.tFRValue, "{0} C".format(round(tFR)))
    ac.setText(self.tRLValue, "{0} C".format(round(tRL)))
    ac.setText(self.tRRValue, "{0} C".format(round(tRR)))
  
  def setPressure(self, pFL, pFR, pRL, pRR):
    ac.setText(self.pFLValue, "{0} psi".format(round(pFL)))
    ac.setText(self.pFRValue, "{0} psi".format(round(pFR)))
    ac.setText(self.pRLValue, "{0} psi".format(round(pRL)))
    ac.setText(self.pRRValue, "{0} psi".format(round(pRR)))

  def setWear(self, wFL, wFR, wRL, wRR):
    ac.setText(self.wFLValue, "{0}%".format(round(wFL)))
    ac.setText(self.wFRValue, "{0}%".format(round(wFR)))
    ac.setText(self.wRLValue, "{0}%".format(round(wRL)))
    ac.setText(self.wRRValue, "{0}%".format(round(wRR)))

  def setMaxT(self, tFL, tFR, tRL, tRR):
    if self.maxtFL < tFL:
      self.maxtFL = tFL
      ac.setText(self.maxtFLValue, "{0}".format(round(self.maxtFL)))
    if self.maxtFR < tFR:
      self.maxtFR = tFR
      ac.setText(self.maxtFRValue, "{0}".format(round(self.maxtFR)))
    if self.maxtRL < tRL:
      self.maxtRL = tRL
      ac.setText(self.maxtRLValue, "{0}".format(round(self.maxtRL)))
    if self.maxtRR < tRR:
      self.maxtRR = tRR
      ac.setText(self.maxtRRValue, "{0}".format(round(self.maxtRR)))

  def setMaxP(self, pFL, pFR, pRL, pRR):
    if self.maxpFL < pFL:
      self.maxpFL = pFL
      ac.setText(self.maxpFLValue, "{0}".format(round(self.maxpFL)))
    if self.maxpFR < pFR:
      self.maxpFR = pFR
      ac.setText(self.maxpFRValue, "{0}".format(round(self.maxpFR)))
    if self.maxpRL < pRL:
      self.maxpRL = pRL
      ac.setText(self.maxpRLValue, "{0}".format(round(self.maxpRL)))
    if self.maxpRR < pRR:
      self.maxpRR = pRR
      ac.setText(self.maxpRRValue, "{0}".format(round(self.maxpRR)))

def acMain(ac_version):
  #ac.log("tyre_info::acMain")
  global TYREINFO
  appWindow = ac.newApp("Tyre Temperatures")
  ac.setTitle(appWindow, "")
  ac.drawBackground(appWindow, background)  
  ac.drawBorder(appWindow, background)
  ac.setBackgroundOpacity(appWindow, background)
  ac.setIconPosition(appWindow, 0, -9000)
  ac.setSize(appWindow, x_app_size, y_app_size)
  TYREINFO = Tyre_Info(appWindow, x_start, y_start)
  ac.addRenderCallback(appWindow, onFormRender)
  
  spinnerid = ac.addSpinner(appWindow, "")
  ac.setPosition(spinnerid, x_start + 103, y_start + 110) 
  ac.setSize(spinnerid, 60, 20)
  ac.setRange(spinnerid, 0, len(tyres)-1)
  ac.setStep(spinnerid, 1)
  ac.setValue(spinnerid, 0)    
  ac.addOnValueChangeListener(spinnerid, onValueChanged)  
  
  TYREINFO.tyre_label = ac.addLabel(appWindow, "tyre_label")
  ac.setPosition(TYREINFO.tyre_label, x_start + 100, y_start + 130)
  ac.setFontSize(TYREINFO.tyre_label, 10)  
  
  onValueChanged(0)

  ac.log("Danny Giusa Tyre Temperatures loaded")
  return "Danny Giusa Tyre Temperatures"

def onValueChanged(value):
  global TYREINFO  
  t = tyres[value]
  TYREINFO.optimal_temp = (float(t[1]) + float(t[2]))*0.5
  TYREINFO.temp_range = float(t[2]) - float(t[1])
  TYREINFO.tyre_selected = t[0]
  ac.setText(TYREINFO.tyre_label, "%s [%d - %d]" % t)
 
def onFormRender(deltaT):
  global TYREINFO
  tFL, tFR, tRL, tRR = ac.getCarState(0, acsys.CS.CurrentTyresCoreTemp)
  pFL, pFR, pRL, pRR = ac.getCarState(0, acsys.CS.DynamicPressure)
  dFL, dFR, dRL, dRR = ac.getCarState(0, acsys.CS.TyreDirtyLevel)
  wFL, wFR, wRL, wRR = readTyreWear()
  TYREINFO.setTemp(tFL, tFR, tRL, tRR)
  TYREINFO.setPressure(pFL, pFR, pRL, pRR)
  TYREINFO.setWear(wFL, wFR, wRL, wRR)
  TYREINFO.setMaxT(tFL, tFR, tRL, tRR)
  TYREINFO.setMaxP(pFL, pFR, pRL, pRR)
  drawTyresAll(w_tyre, h_tyre, round(tFL, 4), round(tFR, 4), round(tRL, 4), round(tRR, 4), dFL, dFR, dRL, dFR)

def drawTyresAll(w, h, tFL, tFR, tRL, tRR, dFL, dFR, dRL, dRR):
  drawTyres(x_tyre_position, y_tyre_position, w, h, tFL)
  drawTyres(x_tyre_position + x_space - x_tyre_space, y_tyre_position, w, h, tFR)
  drawTyres(x_tyre_position, y_tyre_position + y_space, w, h, tRL)
  drawTyres(x_tyre_position + x_space - x_tyre_space, y_tyre_position + y_space, w, h, tRR)
  drawTyresDirt(x_tyre_position + x_tyredirt_space, y_tyre_position + h_tyre, w_tyredirt, 0, dFL)
  drawTyresDirt(x_tyre_position + x_space - x_tyre_space + x_tyredirt_space, y_tyre_position + h_tyre, w_tyredirt, 0, dFR)
  drawTyresDirt(x_tyre_position + x_tyredirt_space, y_tyre_position + y_space + h_tyre, w_tyredirt, 0, dRL)
  drawTyresDirt(x_tyre_position + x_space - x_tyre_space + x_tyredirt_space, y_tyre_position + y_space + h_tyre, w_tyredirt, 0, dRR)

def readTyreWear():
  shmHandle = mmap.mmap(0, 256, "acpmf_physics")
  shmHandle.seek(30*4)
  data = shmHandle.read(4*4)
  shmHandle.close()  
  return struct.unpack("<ffff", data)

def drawTyres(x, y, w, h, temp):
  colorTyres(temp)
  ac.glQuad(x, y, w, h)
   
def drawTyresDirt(x, y, w, h, dirt):
  ac.glColor4f(0.9, 0.6, 0.2, 1)
  ac.glQuad(x, y, w, -(10 * dirt))
   
def colorTyres(temp):
  global TYREINFO
  min_temp = (TYREINFO.optimal_temp - TYREINFO.temp_range/2.)
  max_temp = (TYREINFO.optimal_temp + TYREINFO.temp_range/2.)
  blue = 240
  cyan = 180
  yellow = 60
  red = 0
  
  # mapping m*temp + t such that 
  #   m*min_temp + t = cyan (180)
  #   m*max_temp + t = yellow  (60)
  # t = cyan-m*min_temp
  # m*max_temp + cyan-m*min_temp = yellow
  # m*(max_temp-min_temp) = yellow - cyan
  # m = (yellow - cyan)/(max_temp-min_temp)
  
  m = (yellow - cyan)/(max_temp-min_temp)
  t = cyan - m*min_temp
  
  hue = m*temp + t
  #ac.log("color tyres %f %f %f %f" % (temp, TYREINFO.optimal_temp, TYREINFO.temp_range, hue))
  
  if hue < red: hue = red
  elif hue > blue: hue = blue
  r, g, b = hsv2rgb(hue, 1, 0.8)
  ac.glColor4f(r, g, b, 1)

def hsv2rgb(h, s, v):
  h = float(h)
  s = float(s)
  v = float(v)
  h60 = h / 60.0
  h60f = math.floor(h60)
  hi = int(h60f) % 6
  f = h60 - h60f
  p = v * (1 - s)
  q = v * (1 - f * s)
  t = v * (1 - (1 - f) * s)
  r, g, b = 0, 0, 0
  if hi == 0: r, g, b = v, t, p
  elif hi == 1: r, g, b = q, v, p
  elif hi == 2: r, g, b = p, v, t
  elif hi == 3: r, g, b = p, q, v
  elif hi == 4: r, g, b = t, p, v
  elif hi == 5: r, g, b = v, p, q  
  return r, g, b